/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
*
*  MODULE :             Linked Element
*  SCOPE :              Implements list of linked elements
*  FILE :               linkedlist.hxx
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma interface
#endif
#ifndef _linkedlist_HXX
#define _linkedlist_HXX

/* IMPORTS *******************************************************************/

#ifndef _stdlib_H
#define _stdlib_H
#include <stdlib.h>
#endif

#ifndef __stdio_H
#define __stdio_H
#include <stdio.h>
#endif


/*****************************************************************************/

class LinkedElem {
friend class LinkedList ;
friend class IteratorLinkedList ;

  public :  

  LinkedElem() : _next(NULL), _prev(NULL) {}
  ~LinkedElem() { this->remove() ; }

  void insertAfter(LinkedElem* el) {
    this->_next = el->_next ;
    this->_prev = el ;
    el->_next->_prev = this ;
    el->_next = this ;
  }
  
  void insertBefore(LinkedElem* el) {
    this->_next = el ;
    this->_prev = el->_prev ;
    el->_prev->_next = this ;
    el->_prev = this ;
  }
  
  void remove() {
    if (_next != NULL) _next->_prev = _prev ;
    if (_prev != NULL) _prev->_next = _next ;
    _next = _prev = NULL ;
  }

  LinkedElem* next() {
    return ( (_next == NULL)||(_next->_next == NULL)) ? NULL : _next ;
  }
  LinkedElem* prev() {
    return ( (_prev == NULL) || (_prev->_prev == NULL)) ? NULL : _prev ;
  }
  
  private :

  LinkedElem *_next, *_prev ;
} ;



class LinkedList {
friend class IteratorLinkedList ;
  
  public :

  LinkedList() ;

  void insertHead(LinkedElem* el) ;
  void insertTail(LinkedElem* el) ;
  LinkedElem* head() { return _firstBumper.next() ; }
  LinkedElem* tail() { return _lastBumper.prev() ; }
  
  private :

  LinkedElem _firstBumper, _lastBumper ;
} ;
       

class IteratorLinkedList {

  public :
  
  IteratorLinkedList(LinkedList& l) : list(l), curr(l._firstBumper._next) {} ;

  int more() {
    return ((curr!=&list._firstBumper)&&(curr!=&list._lastBumper)) ; }
  LinkedElem* get() {
    return ((curr==&list._firstBumper)||(curr==&list._lastBumper))?NULL:curr;
  }
  void  next() { if (curr != &list._lastBumper) curr = curr->_next ; }
  void  prev() { if (curr != &list._firstBumper) curr = curr->_prev ; }

  private :

  LinkedList list ;
  LinkedElem* curr ;
} ;

#endif
